package ru.landgrafhomyak.serdha.api.v0.ddl

import ru.landgrafhomyak.serdha.api.v0.dml.Delete
import ru.landgrafhomyak.serdha.api.v0.dml.DeleteCreator
import ru.landgrafhomyak.serdha.api.v0.dml.Insert
import ru.landgrafhomyak.serdha.api.v0.dml.InsertCreator
import ru.landgrafhomyak.serdha.api.v0.dml.Select
import ru.landgrafhomyak.serdha.api.v0.dml.SelectCreator
import ru.landgrafhomyak.serdha.api.v0.dml.Update
import ru.landgrafhomyak.serdha.api.v0.dml.UpdateCreator

public interface ModuleCreator {
	public interface CreateTable<TableUserExtension : Any> {
		public fun createTable(creator: TableCreator<TableUserExtension>): TableUserExtension
	}

	public fun <TableUserExtension : Any> createTable(namespace: Namespace, name: String, initializer: CreateTable<TableUserExtension>): Table<TableUserExtension, Nothing>

	public interface UpdateTable<TableNewUserExtension : Any, TableOldUserExtension : Any> {
		public fun updateTable(oldTable: Table<TableOldUserExtension, *>, updater: TableUpdater<TableNewUserExtension, TableOldUserExtension>): TableNewUserExtension
	}

	public fun <TableNewUserExtension : Any, TableOldUserExtension : Any> updateTable(
		oldTable: Table<TableOldUserExtension, *>,
		initializer: UpdateTable<TableNewUserExtension, TableOldUserExtension>
	): Table<TableNewUserExtension, TableOldUserExtension>

	public fun <TableNewUserExtension : Any, TableOldUserExtension : Any> renameTable(
		table: Table<TableNewUserExtension, TableOldUserExtension>,
		newName: String
	): Table<TableNewUserExtension, TableOldUserExtension>

	public fun <TableNewUserExtension : Any, TableOldUserExtension : Any> renameTable(
		table: Table<TableNewUserExtension, TableOldUserExtension>,
		newNamespace: Namespace,
		newName: String
	): Table<TableNewUserExtension, TableOldUserExtension>

	public fun <TableNewUserExtension : Any, TableOldUserExtension : Any> updateAndRenameTable(
		table: Table<TableOldUserExtension, *>,
		newName: String,
		initializer: UpdateTable<TableNewUserExtension, TableOldUserExtension>
	): Table<TableNewUserExtension, TableOldUserExtension>

	public fun <TableNewUserExtension : Any, TableOldUserExtension : Any> updateAndRenameTable(
		table: Table<TableNewUserExtension, TableOldUserExtension>,
		newNamespace: Namespace,
		newName: String,
		initializer: UpdateTable<TableNewUserExtension, TableOldUserExtension>
	): Table<TableNewUserExtension, TableOldUserExtension>

	public interface CreateSelect<QueryUserWrapper : Any> {
		public fun createSelect(creator: SelectCreator<QueryUserWrapper>): QueryUserWrapper
	}

	public fun <QueryUserWrapper : Any> createSelect(initializer: CreateSelect<QueryUserWrapper>): Select<QueryUserWrapper>

	public interface CreateInsert<TableUserExtension : Any, QueryUserWrapper : Any> {
		public fun createInsert(table: Table<TableUserExtension, *>, creator: InsertCreator<TableUserExtension, QueryUserWrapper>): QueryUserWrapper
	}

	public fun <TableUserExtension : Any, QueryUserWrapper : Any> createInsert(table: Table<TableUserExtension, *>, initializer: CreateInsert<TableUserExtension, QueryUserWrapper>): Insert<QueryUserWrapper>

	public interface CreateUpdate<TableUserExtension : Any, QueryUserWrapper : Any> {
		public fun createUpdate(table: Table<TableUserExtension, *>, creator: UpdateCreator<TableUserExtension, QueryUserWrapper>): QueryUserWrapper
	}

	public fun <TableUserExtension : Any, QueryUserWrapper : Any> createUpdate(table: Table<TableUserExtension, *>, initializer: CreateUpdate<TableUserExtension, QueryUserWrapper>): Update<QueryUserWrapper>

	public interface CreateDelete<TableUserExtension : Any, QueryUserWrapper : Any> {
		public fun createDelete(table: Table<TableUserExtension, *>, creator: DeleteCreator<TableUserExtension, QueryUserWrapper>): QueryUserWrapper
	}

	public fun <TableUserExtension : Any, QueryUserWrapper : Any> createDelete(table: Table<TableUserExtension, *>, initializer: CreateDelete<TableUserExtension, QueryUserWrapper>): Delete<QueryUserWrapper>

	public fun <ModuleUserExtension : Any> substituteModule(rootNs: Namespace, template: ModuleTemplate<ModuleUserExtension>): Module<ModuleUserExtension>
	public fun <ModuleUserExtension : Any> upgradeModule(oldModule: Module<*>, rootNs: Namespace, template: ModuleTemplate<ModuleUserExtension>): Module<ModuleUserExtension>
}